#include <stdlib.h>
#include <LiquidCrystal.h>

volatile int brake_count = 0;
volatile unsigned long brake_time;
volatile unsigned long engine_time;
volatile unsigned long last_engine_time;
volatile unsigned long revolutions;

volatile bool brake;
volatile bool stopped;   // Not used yet

const char tab('\t');

const int EnginePin = 3;
const int BrakePin  = 2;

float TORSION = 0.00004;  // Calibration Value

// Liquid Crystal Display constants and pin connections
// This pin setup is for a DF Robot LCD shield.  
// Other 1802 LCDs use other pins.
// If you use other pins note that pins 2 & 3 are in use!

const int LCD_cols = 16;  // Must be 16 or 20 columns. Depends on the LCD used
const int LCD_rows = 2;   // Must be 2 or 4 rows
LiquidCrystal lcd(8,9,4,5,6,7); 

void setup() {
  Serial.begin( 57600 );
 
  // initialise the LCD
  lcd.begin( LCD_cols, LCD_rows );
  lcd.clear();
  
  pinMode( EnginePin, INPUT );
  pinMode( BrakePin,  INPUT );

  attachInterrupt( digitalPinToInterrupt( EnginePin ), engine_event, FALLING );
  attachInterrupt( digitalPinToInterrupt( BrakePin ), brake_event, FALLING );
  brake = false;
  stopped = true;
  revolutions = 0;
  brake_count = 0;
  last_engine_time = engine_time = millis();
  Serial.println( "time\trpm\trev_t\tbrakes\trevs\tbrake_t\tangle\tpower" );
}

void loop() {
  long revolution_time;
  unsigned int rpm, corrected_rpm;
  float angle = 0.0, power = 0.0;
  char angle_s[32];
  char line1[16], line2[16];

  if ( !stopped )
  {
    revolution_time = (long) engine_time - (long)last_engine_time;
    rpm = ( revolution_time != 0 ) ? 600000 / revolution_time : 0 ;  // RPM * 10
    if ( rpm > 0 )
    {
      if ( brake )
      {
        angle =  360 + ( (long) brake_time - (long)engine_time ) * 360.0 / revolution_time;  
      }
      else
        angle = 0.0;

      brake = false;
      corrected_rpm = rpm / 10;  // rpm is an integer * 10. Convert to actual rpm, rounding up
      if ( rpm % 10 > 4 )         
        corrected_rpm++;
      
      dtostrf( angle, 6, 2, angle_s );
      power = TORSION * TWO_PI * corrected_rpm / 60 * 1000; //mW
  
      Serial.print( millis() );
      Serial.print( tab );
      Serial.print( corrected_rpm );
      Serial.print( tab );
      Serial.print( revolution_time );
      Serial.print( tab );
      Serial.print( brake_count );
      Serial.print( tab );
      Serial.print( revolutions );
      Serial.print( tab );
      Serial.print( (long) engine_time - (long) brake_time );
      Serial.print( tab );
      Serial.print( angle_s );
      Serial.print( tab );
      Serial.println( power );
      if ( corrected_rpm > 999 )
        corrected_rpm = 0;
      
      lcd.setCursor( 0,0 );
      sprintf(line1,"%3urpm %08.6s\xdf", corrected_rpm, angle_s);
      lcd.print( line1 );
      lcd.setCursor( 0,1 );
      lcd.print( power );
      lcd.print( " mW");
    }
  }
  stopped = true;
}

void engine_event()
{
  // Interrupt Service Routine.
  last_engine_time = engine_time;
  engine_time = millis();
  revolutions++;
  stopped = false;
}

void brake_event()
{
  // Interrupt Service Routine.
  brake_count++;
  brake_time = millis();
  brake = true;
}





